"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeScriptAdapter = void 0;
const common_1 = require("@typescript-nameof/common");
/**
 * Provides the functionality to parse and dump `nameof` calls for typescript.
 */
class TypeScriptAdapter extends common_1.Adapter {
    /**
     * Initializes a new instance of the {@linkcode TypeScriptAdapter} class.
     *
     * @param features
     * The features of the platform integration.
     */
    constructor(features) {
        super(features);
    }
    /**
     * Gets the TypeScript compiler instance.
     */
    get TypeScript() {
        return this.Features.TypeScript;
    }
    /**
     * Gets a component for printing TypeScript nodes.
     */
    get Printer() {
        if (this.printer === undefined) {
            this.printer = this.TypeScript.createPrinter();
        }
        return this.printer;
    }
    /**
     * @inheritdoc
     *
     * @param input
     * The input to extract the node from.
     *
     * @returns
     * The node that was extracted from the specified {@linkcode input}.
     */
    Extract(input) {
        return input;
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item whose location to get.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The location of the specified {@linkcode item}.
     */
    GetLocation(item, context) {
        let location = context.file.getLineAndCharacterOfPosition(item.getStart(context.file));
        return {
            filePath: context.file.fileName,
            line: location.line,
            column: location.character
        };
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item to get the source code from.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The code of the specified {@linkcode item}.
     */
    GetSourceCode(item, context) {
        return item.getText(context.file);
    }
    /**
     * Prints the code of the specified {@linkcode item}.
     *
     * @param item
     * The item to print.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The generated code of the specified {@linkcode item}.
     */
    PrintSourceCode(item, context) {
        return this.Printer.printNode(this.TypeScript.EmitHint.Unspecified, item, this.TypeScript.createSourceFile("typescript-nameof.ts", "", this.TypeScript.ScriptTarget.ES2022));
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a call expression.
     */
    IsCallExpression(item) {
        return this.TypeScript.isCallExpression(item);
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a string literal.
     */
    IsStringLiteral(item) {
        return this.TypeScript.isStringLiteral(item);
    }
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a template literal.
     */
    IsTemplateLiteral(item) {
        return this.TypeScript.isTemplateLiteral(item);
    }
    /**
     * Gets the elements of the specified {@linkcode arrayLiteral}.
     *
     * @param arrayLiteral
     * The array literal to get the elements from.
     *
     * @returns
     * Either the elements of the {@linkcode arrayLiteral} or `undefined` if the specified {@linkcode arrayLiteral} is invalid.
     */
    GetArrayElements(arrayLiteral) {
        if (this.TypeScript.isArrayLiteralExpression(arrayLiteral)) {
            let result = [];
            for (let element of arrayLiteral.elements) {
                if (element) {
                    result.push(element);
                }
            }
            return result;
        }
        else {
            return undefined;
        }
    }
    /**
     * @inheritdoc
     *
     * @param elements
     * The elements of the array literal to create.
     *
     * @returns
     * The newly created array literal.
     */
    CreateArrayLiteral(elements) {
        return this.TypeScript.factory.createArrayLiteralExpression(elements);
    }
    /**
     * Parses the specified {@linkcode item}.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item}.
     */
    ParseInternal(item, context) {
        var _a;
        if (this.IsCallExpression(item)) {
            return new common_1.CallExpressionNode(item, item.expression, (_a = item.typeArguments) !== null && _a !== void 0 ? _a : [], item.arguments);
        }
        else if ([
            this.TypeScript.SyntaxKind.ThisKeyword,
            this.TypeScript.SyntaxKind.SuperKeyword,
            this.TypeScript.SyntaxKind.AnyKeyword,
            this.TypeScript.SyntaxKind.UnknownKeyword,
            this.TypeScript.SyntaxKind.VoidKeyword,
            this.TypeScript.SyntaxKind.NeverKeyword,
            this.TypeScript.SyntaxKind.ObjectKeyword,
            this.TypeScript.SyntaxKind.BooleanKeyword,
            this.TypeScript.SyntaxKind.NumberKeyword,
            this.TypeScript.SyntaxKind.BigIntKeyword,
            this.TypeScript.SyntaxKind.StringKeyword,
            this.TypeScript.SyntaxKind.SymbolKeyword
        ].includes(item.kind)) {
            return new common_1.IdentifierNode(item, this.PrintSourceCode(item, context));
        }
        else if (this.TypeScript.isNumericLiteral(item)) {
            return new common_1.NumericLiteralNode(item, parseFloat(item.text));
        }
        else if (this.IsStringLiteral(item)) {
            return new common_1.StringLiteralNode(item, item.text);
        }
        else if (this.TypeScript.isIdentifier(item)) {
            return new common_1.IdentifierNode(item, this.PrintSourceCode(item, context));
        }
        else if (this.TypeScript.isTypeReferenceNode(item)) {
            return this.ParseNode(item.typeName, context);
        }
        else if (this.TypeScript.isImportTypeNode(item)) {
            if (item.qualifier) {
                return this.ParseNode(item.qualifier, context);
            }
            else {
                throw new common_1.MissingImportTypeQualifierError(this, item, context);
            }
        }
        else if (this.TypeScript.isNonNullExpression(item) ||
            this.TypeScript.isParenthesizedExpression(item) ||
            this.TypeScript.isAsExpression(item)) {
            return this.ParseNode(item.expression, context);
        }
        else if (this.TypeScript.isPrefixUnaryExpression(item)) {
            if ([
                this.TypeScript.SyntaxKind.PlusToken,
                this.TypeScript.SyntaxKind.MinusToken
            ].includes(item.operator)) {
                let node = this.ParseNode(item.operand, context);
                if (node.Type === "NumericLiteralNode" /* NodeKind.NumericLiteralNode */) {
                    return new common_1.NumericLiteralNode(item, node.Value * (item.operator === this.TypeScript.SyntaxKind.MinusToken ? -1 : 1));
                }
            }
        }
        else if (this.TypeScript.isPropertyAccessExpression(item)) {
            return this.ParsePropertyAccessExpression(item, item.expression, item.name, context);
        }
        // Dotted paths in type references
        else if (this.TypeScript.isQualifiedName(item)) {
            return this.ParsePropertyAccessExpression(item, item.left, item.right, context);
        }
        else if (this.TypeScript.isElementAccessExpression(item)) {
            return new common_1.IndexAccessNode(item, this.ParseNode(item.expression, context), this.ParseNode(item.argumentExpression, context));
        }
        // Index access paths in type references
        else if (this.TypeScript.isIndexedAccessTypeNode(item)) {
            let indexer;
            if (this.TypeScript.isLiteralTypeNode(item.indexType)) {
                indexer = item.indexType.literal;
            }
            else {
                indexer = item.indexType;
            }
            return new common_1.IndexAccessNode(item, this.ParseNode(item.objectType, context), this.ParseNode(indexer, context));
        }
        else if (this.TypeScript.isArrowFunction(item) ||
            this.TypeScript.isFunctionExpression(item)) {
            return this.ParseFunction(item, context);
        }
        return new common_1.UnsupportedNode(item);
    }
    /**
     * Parses a {@linkcode PropertyAccessNode} with the specified {@linkcode expression} and {@linkcode property}.
     *
     * @param source
     * The source of the node.
     *
     * @param expression
     * The expression of the property access expression.
     *
     * @param property
     * The property that is accessed in the expression.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode source}.
     */
    ParsePropertyAccessExpression(source, expression, property, context) {
        return new common_1.PropertyAccessNode(source, this.ParseNode(expression, context), this.PrintSourceCode(property, context));
    }
    /**
     * Parses the specified {@linkcode item}.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item}.
     */
    ParseFunction(item, context) {
        let expression = this.GetReturnExpression(item.body);
        if (expression) {
            return new common_1.FunctionNode(item, item.parameters.map((parameter) => {
                if (this.TypeScript.isIdentifier(parameter.name)) {
                    return this.PrintSourceCode(parameter.name, context);
                }
                else {
                    throw new common_1.UnsupportedNodeError(this, parameter, context);
                }
            }), expression);
        }
        else {
            throw new common_1.NoReturnExpressionError(this, item, context);
        }
    }
    /**
     * Gets the return expression from the specified {@linkcode body}.
     *
     * @param body
     * The function body to get the return statement from.
     *
     * @returns
     * The expression in the return statement of the specified {@linkcode body}.
     */
    GetReturnExpression(body) {
        if (this.TypeScript.isBlock(body)) {
            for (let statement of body.statements) {
                if (this.TypeScript.isReturnStatement(statement)) {
                    if (statement.expression) {
                        return statement.expression;
                    }
                }
                else {
                    // ToDo: Throw error regarding non-return statement
                }
            }
            return undefined;
        }
        else {
            return body;
        }
    }
    /**
     * Dumps the specified {@linkcode item}.
     *
     * @param item
     * The item to dump.
     *
     * @returns
     * The dumped node.
     */
    Dump(item) {
        switch (item.type) {
            case common_1.ResultType.Plain:
                return this.TypeScript.factory.createStringLiteral(item.text);
            case common_1.ResultType.Template:
                return this.TypeScript.factory.createTemplateExpression(this.TypeScript.factory.createTemplateHead(item.templateParts[0]), item.expressions.map((expression, index) => {
                    let spanIndex = index + 1;
                    let text = item.templateParts[spanIndex];
                    let templatePart;
                    if (spanIndex === item.expressions.length) {
                        templatePart = this.TypeScript.factory.createTemplateTail(text);
                    }
                    else {
                        templatePart = this.TypeScript.factory.createTemplateMiddle(text);
                    }
                    return this.TypeScript.factory.createTemplateSpan(expression, templatePart);
                }));
            case common_1.ResultType.Node:
                return item.node;
        }
    }
}
exports.TypeScriptAdapter = TypeScriptAdapter;
//# sourceMappingURL=TypeScriptAdapter.cjs.map