import { Adapter, FunctionNode, INodeLocation, NameofResult, ParsedNode, PropertyAccessNode } from "@typescript-nameof/common";
import ts = require("typescript");
import { ITypeScriptContext } from "./ITypeScriptContext.cjs";
import { TypeScriptFeatures } from "./TypeScriptFeatures.cjs";
/**
 * Provides the functionality to parse and dump `nameof` calls for typescript.
 */
export declare class TypeScriptAdapter extends Adapter<TypeScriptFeatures, ts.Node, ts.Node, ITypeScriptContext> {
    /**
     * A component for printing TypeScript nodes.
     */
    private printer;
    /**
     * Initializes a new instance of the {@linkcode TypeScriptAdapter} class.
     *
     * @param features
     * The features of the platform integration.
     */
    constructor(features: TypeScriptFeatures);
    /**
     * Gets the TypeScript compiler instance.
     */
    get TypeScript(): typeof ts;
    /**
     * Gets a component for printing TypeScript nodes.
     */
    get Printer(): ts.Printer;
    /**
     * @inheritdoc
     *
     * @param input
     * The input to extract the node from.
     *
     * @returns
     * The node that was extracted from the specified {@linkcode input}.
     */
    Extract(input: ts.Node): ts.Node;
    /**
     * @inheritdoc
     *
     * @param item
     * The item whose location to get.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The location of the specified {@linkcode item}.
     */
    GetLocation(item: ts.Node, context: ITypeScriptContext): INodeLocation;
    /**
     * @inheritdoc
     *
     * @param item
     * The item to get the source code from.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The code of the specified {@linkcode item}.
     */
    GetSourceCode(item: ts.Node, context: ITypeScriptContext): string;
    /**
     * Prints the code of the specified {@linkcode item}.
     *
     * @param item
     * The item to print.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The generated code of the specified {@linkcode item}.
     */
    PrintSourceCode(item: ts.Node, context: ITypeScriptContext): string;
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a call expression.
     */
    protected IsCallExpression(item: ts.Node): item is ts.CallExpression;
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a string literal.
     */
    protected IsStringLiteral(item: ts.Node): item is ts.StringLiteral;
    /**
     * @inheritdoc
     *
     * @param item
     * The item to check.
     *
     * @returns
     * A value indicating whether the specified {@linkcode item} is a template literal.
     */
    protected IsTemplateLiteral(item: ts.Node): boolean;
    /**
     * Gets the elements of the specified {@linkcode arrayLiteral}.
     *
     * @param arrayLiteral
     * The array literal to get the elements from.
     *
     * @returns
     * Either the elements of the {@linkcode arrayLiteral} or `undefined` if the specified {@linkcode arrayLiteral} is invalid.
     */
    protected GetArrayElements(arrayLiteral: ts.Node): ts.Node[] | undefined;
    /**
     * @inheritdoc
     *
     * @param elements
     * The elements of the array literal to create.
     *
     * @returns
     * The newly created array literal.
     */
    protected CreateArrayLiteral(elements: ts.Node[]): ts.Node;
    /**
     * Parses the specified {@linkcode item}.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item}.
     */
    protected ParseInternal(item: ts.Node, context: ITypeScriptContext): ParsedNode<ts.Node>;
    /**
     * Parses a {@linkcode PropertyAccessNode} with the specified {@linkcode expression} and {@linkcode property}.
     *
     * @param source
     * The source of the node.
     *
     * @param expression
     * The expression of the property access expression.
     *
     * @param property
     * The property that is accessed in the expression.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode source}.
     */
    protected ParsePropertyAccessExpression(source: ts.Node, expression: ts.Node, property: ts.MemberName, context: ITypeScriptContext): PropertyAccessNode<ts.Node>;
    /**
     * Parses the specified {@linkcode item}.
     *
     * @param item
     * The item to parse.
     *
     * @param context
     * The context of the operation.
     *
     * @returns
     * The parsed representation of the specified {@linkcode item}.
     */
    protected ParseFunction(item: ts.ArrowFunction | ts.FunctionExpression, context: ITypeScriptContext): FunctionNode<ts.Node>;
    /**
     * Gets the return expression from the specified {@linkcode body}.
     *
     * @param body
     * The function body to get the return statement from.
     *
     * @returns
     * The expression in the return statement of the specified {@linkcode body}.
     */
    protected GetReturnExpression(body: ts.ConciseBody): ts.Node | undefined;
    /**
     * Dumps the specified {@linkcode item}.
     *
     * @param item
     * The item to dump.
     *
     * @returns
     * The dumped node.
     */
    protected Dump(item: NameofResult<ts.Node>): ts.Node;
}
